# @File (label = "image file") afile
# @String (label = "File type", choices = {".czi", ".nd"}, style = "radioButtonHorizontal") filetype
# @Integer (label = "Number of rows (czi)", value = 8) nrRows
# @Integer (label = "Number of columns (czi)", value = 12) nrCols
# @Boolean (label = "Maximal Projection") maxproj
# @Boolean (label = "Alphanumeric output") alphanumeric
# @DatasetIOService io

ftypes = {"Visi":".nd",
          "Zeiss":".czi"}

from ij import IJ
import os
import re
from loci.formats import ImageReader
from loci.plugins import BF
from loci.plugins.in import ImporterOptions
from ij.plugin import RGBStackMerge
from ij.plugin import ZProjector
from net.imglib2.img.display.imagej import ImageJFunctions
from ij.plugin import HyperStackConverter

class maxproj_rename():
    ftypes = {"Visi":".nd",
              "Zeiss":".czi"}
    rownames = ['A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K' 'L', 'M']
    reader = None
    nrseries = None
    maxdigits = None
    outdir = None
    def __init__(self, inputfile, filetype, rows, cols, maxproj, alphanumeric):
        IJ.resetEscape()
        self.filepath = inputfile
        self.filetype = filetype
        self.rows = rows
        self.cols = cols
        self.maxproj = maxproj
        self.alphanumeric = alphanumeric
        extenList = os.path.splitext(os.path.basename(self.filepath))
        self.filename_wo_ext = extenList[0]
        self.extension = extenList[1]


    def assignreader(self):
        if not self.extension == self.filetype:
            msg = "file chosen does not match the extension"
            IJ.showMessage(msg)
            return False
        if self.extension == self.ftypes['Zeiss']:
            if self.nrseries != self.rows*self.cols and self.alphanumeric == True:
                msg = "Total number of images %d does not match expected number %d*%d = %d" %(self.nrseries, self.rows, self.cols, self.rows * self.cols)
                IJ.showMessage(msg)
                return False
        self.reader = ImageReader()
        self.reader.setId(self.filepath)
        self.nrseries = self.reader.getSeriesCount()
        self.maxdigits = len(str(self.nrseries))
        extension = os.path.splitext(os.path.basename(self.filepath))[1]
        self.outdir = os.path.join(os.path.dirname(self.filepath) +"_proc", "tifs")
        if not os.path.exists(self.outdir):
            os.makedirs(self.outdir)
        return True

    def createMergedChannelImage(self, listoffiles = None, seriesidx = 0, nrchannels = 0, calibration  = None):
        if listoffiles is None:
            return
        imgs = list()
        for ic in range(1, nrchannels + 1):
            try:
                img = io.open(listoffiles[nrchannels * seriesidx + ic])
            except AttributeError:
                fname_mod = os.path.splitext(listoffiles[nrchannels * seriesidx + ic])[0] +".stk"
                img = io.open(fname_mod)
            imgP =  ImageJFunctions.wrap(img, "Ch"+str(ic))

            # io often reads the files with NChannels instead of NFrames
            if imgP.getNChannels() > 1:
                imgP = HyperStackConverter.toHyperStack(imgP, 1, imgP.getNChannels(), 1)
            if self.maxproj and imgP.getNFrames > 1:
                imgs.append(ZProjector.run(imgP, "max"))
            else:
                imgs.append(imgP)

        cm = RGBStackMerge()
        if self.maxproj or imgP.getNFrames() == 1:
            imp = cm.mergeChannels(imgs, False)
        else:
            imp = cm.mergeHyperstacks(imgs, False)
        return imp

    def parseNameVisiView(self, seriesname = None, seriesnr = 1):
        """Name is of type row:A,column:1,site:1"""
        m = re.match("\"row:(?P<row>\w+),column:(?P<column>\d+),site:(?P<site>\d+)\"", seriesname)
        print(seriesname)
        if m:
            if self.alphanumeric:
                outStr = m.group('row') + IJ.pad(m.group('column'),2) + "-P" + IJ.pad(m.group('site'),2)
            else:
                outStr = IJ.pad(seriesnr, self.maxdigits)
        else:
            m = re.match("\"Row(?P<row>\w+)_Col(?P<column>\w+)\"", seriesname)
            if m:
                if self.alphanumeric:
                    outStr = "Row" + IJ.pad(m.group('row'),3) + "Col" + IJ.pad(m.group('column'),3)
                else:
                    outStr = IJ.pad(seriesnr, self.maxdigits)
            else:
                outStr = ""

        return outStr

    def parseNameZEN(self, seriesname = None):

        """ Name is of type scene
        :return: A string
        """

        m = re.match( ".+#(?P<scene>\d+)", seriesname)

        #reading from left to right
        scene = int(m.group('scene'))
        arow = self.rownames[(scene-1)/self.nrcols]
        acol = scene - (scene-1)/self.nrcols * self.nrcols

        if m:
            if self.alphanumeric:
                outStr = arow +  IJ.pad(acol, len(str(self.nrcols)))
            else:
                outStr = IJ.pad(m.group('scene'),self.maxdigits)
        else:
            outStr = None

        return outStr

    def processfile_zeiss(self):
        bfOpt = ImporterOptions()
        bfOpt.setId(self.filepath)
        bfOpt.clearSeries()
        IJ.run("Close All")
        for ise in range(0,self.nrseries):
            IJ.showProgress(ise/self.nrseries)
            bfOpt.setSeriesOn(ise, True)
            imp = BF.openImagePlus(bfOpt)[0]
            if IJ.escapePressed():
                return
            outfile = os.path.join(self.outdir, self.filename_wo_ext + "_" + self.parseNameZEN(seriesname = imp.getTitle()) + ".tif")
            IJ.log("Processing position " + self.parseNameZEN(seriesname = imp.getTitle()))
            if self.maxProj:
                impZproj = ZProjector.run(imp, "max")
                IJ.saveAs(impZproj, "Tiff", outfile)
                impZproj.close()
            else:
                IJ.saveAs(imp, "Tiff", outfile)
            bfOpt.setSeriesOn(ise, False)
            imp.close()

    def processfile_visi(self):
        fnames = self.reader.getUsedFiles()
        nrChannels = int(self.reader.getMetadataValue("NWavelengths"))
        bfOpt = ImporterOptions()
        bfOpt.setId(self.filepath)
        bfOpt.clearSeries()
        bfOpt.setSeriesOn(0, True)
        imp = BF.openImagePlus(bfOpt)[0]
        cal = imp.getCalibration()
        bfOpt.setSeriesOn(0, False)
        imp.close()
        IJ.run("Close All")

        for ise in range(0, self.nrseries):
            IJ.showProgress(ise/self.nrseries)
            namePos = self.reader.getMetadataValue("Stage"+str(ise+1))
            #imgnames = list()
            if IJ.escapePressed():
                return
            imp = self.createMergedChannelImage(listoffiles = fnames, seriesidx = ise, nrchannels = nrChannels, calibration = cal)
            imp.setCalibration(cal)
            postfix = self.parseNameVisiView(seriesname = namePos,  seriesnr = ise+1)
            IJ.log("Processing position " + postfix)
            outfile = os.path.join(self.outdir, self.filename_wo_ext + "_" +
                                   postfix + ".tif")
            IJ.saveAs(imp, "Tiff", outfile)
    #imp.close()

    def processfile(self):
        if self.filetype == self.ftypes['Visi']:
            self.processfile_visi()
        if self.filetype == self.ftypes['Zeiss']:
            self.processfile_zeiss()

maxprojC = maxproj_rename(inputfile = afile.getAbsolutePath(), alphanumeric = alphanumeric,
                          cols = nrCols, rows = nrRows, filetype = filetype, maxproj = maxproj)

if maxprojC.assignreader():
    maxprojC.processfile()
else:
    IJ.showMessage("Could not start")
